#!/usr/bin/env stack
-- stack --stack-yaml /localscratch/tct/tctac/stack.yaml runghc --package tctac -- -threaded

{-# OPTIONS_GHC -fno-warn-missing-signatures #-}

import Runner
import Summer

pwd = "/localscratch/tct/paicc-experiments/18dice/"

e = Experiment
  { eTestbed   = pwd ++ "complexity_its_termcomp2017/"
  , eIgnore    = pwd ++ "complexity_its_termcomp2017/"
  , eProcesses = 4
  , eTimeout   = 60
  , eRepeat    = False
  , eTools     =
    [ cvs "cofloco"
    , cvs "aprove"
    , cage "cage-no-chain" ["-max-chaining","0"]

    , paicc "loopstructure" ""
    , pccai "loopstructure" ""
    , pccai "simple" ""
    , paicc "strategic" ""
    , pccai "strategic" ""
    , pccai "unlimited" ""
    ]
  }

rts n = words $ "+RTS -N" ++ show n ++ " -RTS"

cvs n = Tool
  { tName      = n
  , tExtension = "koat"
  , tCommand   = "./fromcvs.sh"
  , tArguments = [pwd ++ "complexity-its.csv", n]
  , tProcessor = \out -> if firstLine out == Success "Nothing" then Maybe else termcomp out
  }

cage n as = Tool
  { tName      = n
  , tExtension = "koat"
  , tCommand   = "./cage.sh"
  , tArguments = ["--use-termcomp-format", "--timeout", "60"] ++ as
  , tProcessor = adapt1 . dropO . termcomp . firstLine }

cige n as = let t = cage n as in t { tName = tName t ++ "-ai", tExtension = "koat-ai"}

paicc n v = Tool
  { tName      = n ++ v
  , tExtension = "koat"
  , tCommand   = "./paicc.sh"
  , tArguments = rts 6 ++ ["-t62","-s"] ++ [n]
  , tProcessor = adapt1 . adaptM . dropO' . dropYES . firstLine
  }

pccai n v = let t = paicc n v in t { tName = tName t ++ "-ai", tExtension = "koat-ai" }

dropYES (Success ('Y':'E':'S':'(':'?':',':xs)) = Success (init $ dropWhile (==' ') xs)
dropYES out                            = out

dropO (Success (' ':'O':'(':xs)) = Success (init xs)
dropO out                        = out

dropO' (Success ('O':'(':xs)) = Success (init xs)
dropO' out                    = out

adapt1(Success "1") = Success "n^0"
adapt1 out          = out

adaptM (Success "MAYBE") = Success "Maybe"
adaptM out               = out

rank "n^0"     = 0
rank "n^1"     = 10
rank "n^2"     = 20
rank "n^3"     = 30
rank "n^4"     = 40
rank ('n':xs)  = 100
rank "POLY"    = 200
rank "PRIMREC" = 600
rank "Maybe"   = 800
rank "MAYBE"   = 800
rank _         = 999

cmp (Success a) (Success b) = rank a `compare` rank b
cmp (Success a) _           = LT
cmp Maybe Maybe             = EQ
cmp Timeout Timeout         = EQ
cmp Maybe _                 = LT
cmp Timeout _               = LT
cmp _ _                     = EQ

main :: IO ()
main = do
  log "BEGIN"
  log "run"
  run e
  log "sumarise"
  summarise (tName <$> eTools e)
  putStrLn "diffby"
  diffBy cmp "strategic-ai" "simple-ai"
  putStrLn "END"
  where log xs = putStrLn $ '[' : xs ++ "]"

